//This is the main file used to generate a text file for
// the master thesis by Rennemo and Roe
//used in StochasticModel_masterfil and 
//DeterministicModel_masterfil 

//variables = all_small_letters
//functions = StartWithBigAndNoUnderline()
//Classes = Only first letter big 
#include "main_functions.h"
#include "location.h"
#include "ICD.h"
#include "stochastic.h"
#include <time.h>
#include <ctime> 
#include <vector>
#include <iostream>
#include <cstring>
#include <cstdlib>
#include <vector>
#include <map>
#include <math.h>

using namespace std;
   

int main () 
{
	//srand(3000000);
	srand(time(0) );

//////// Creating the Stochastic set:
		Stochastic stocset;
		CreateStochastic(stocset);
//Creating the earthquake
		Location earthquake;
		CreateEarthquake(earthquake);

/// Creating the ICDs
		vector<ICD>icder;
		icder.reserve(stocset.GetNumberOfICDs()+1);
		for (int i = 0; i < stocset.GetNumberOfICDs(); i++)
		{
			ICD icd;
			CreateStochICDs(icd, i+1, stocset);
			icder.push_back(icd);
		}
		
		//Creates a map to hold track of indexes
		map<int, ICD>icd_index;
		for (int i = 0 ; i < stocset.GetNumberOfICDs(); i++)
		{
			icd_index[i+1]=icder[i];
		}

////////Creating the LDCs
	vector<LDC>ldcer;
	ldcer.reserve(stocset.GetNumberOfLDCs()+1);
		
		for (int i = 0; i < stocset.GetNumberOfLDCs(); i++)
		{
			LDC ldc;
			CreateStochLDCs(ldc, earthquake, stocset, i+1);
			ldcer.push_back(ldc);
		}

		//Creates a map to hold track of the indexes
		map<int, LDC>ldc_index;
		for (int i = 0 ; i < stocset.GetNumberOfLDCs(); i++)
		{
			ldc_index[i+1+stocset.GetNumberOfICDs()] = ldcer[i];
		}
	ldcer.clear();

/////Creating the PODs
		vector<PODD>podder;
		podder.reserve(stocset.GetNumberOfPODs()+1);
		
		for (int m = 0 ; m < stocset.GetNumberOfPODs() ; m++)
		{
			PODD podd;
			vector<double>row6;
			row6.assign(stocset.GetNumberOfScenarioTreeNodes(), 0);
			
			for (int i = 0 ; i < stocset.GetNumberOfCommodities() ; i++)
			{
				podd.demand.push_back(row6); 
			}

			double indemand;
			cout << "For POD "<< m+1 << ", what is the assumed demand for" << endl;
			for (int i = 0 ; i < stocset.GetNumberOfCommodities() ; i++)
			{
					cout << "Commodity " << i+1 << "           : ";
					cin >> indemand;
					podd.demand[i][0] = indemand;
			}
			
			cout << "The true value of demand is 30% up or down from assumed" << endl;
			
			//	// creating 2nd and 3rd stage actually demand:

			int sub3, sub2;
			sub2 = stocset.GetNumberOfPossibilitiesIn2();
			sub3 = stocset.GetNumberOfPossibilitiesIn3();
			for (int  i = 0 ; i < stocset.GetNumberOfCommodities(); i++)
			{
				for (int j = 0 ; j < stocset.GetNumberOfPossibilitiesIn2() ; j++)
				{
					podd.demand[i][j+1] = CreateActualDemand(podd.demand[i][0]);
					
					for (int k = 0 ; k < stocset.GetNumberOfPossibilitiesIn3() ; k++)
					{
						podd.demand[i][1+sub2 +k +j* sub3] = podd.demand[i][j+1];
					}
				}
			}
			
			int a = earthquake.GetKvadrant();
			podd.SetCoordinates(RandomForKvadrantX(a), RandomForKvadrantY(a));
			
			for ( int i = 0 ; i < stocset.GetNumberOfCommodities() ; i++) 
			{
				podd.ut_interval_size[i][0] = podd.demand[i][0] * 0.2*1.3;
				podd.ut_interval_size[i][1] = podd.demand[i][0] * 0.3*1.3;
				podd.ut_interval_size[i][2] = podd.demand[i][0] * 0.5*1.3;
				podd.ut_interval_size[i][3] = 0;
			
			}
			
			double disx, disy, disxy; 
			disx = podd.GetCoordinateX() - earthquake.GetCoordinateX() ;
			disy = podd.GetCoordinateY() - earthquake.GetCoordinateY();
			disxy = sqrt((disx*disx) + (disy*disy));

			for (int iii = 0; iii < stocset.GetNumberOfCommodities(); iii++)
			{
				for (int jjj = 0 ; jjj < stocset.GetNumberOfScenarioTreeNodes() ; jjj++)
				{
					podd.ut_factor[iii][0][jjj] = (1/disxy) *92000;
					podd.ut_factor[iii][1][jjj] = (1/disxy) *92000*2/3 ;
					podd.ut_factor[iii][2][jjj] = (1/disxy) *92000/3 ;
					podd.ut_factor[iii][3][jjj] = 0;
				}
			}
	
			podder.push_back(podd);
		}
		

		//srand(time(0) );
//////Creating the links between 
		Arcs arcs;

		double radius = sqrt(float((650*650)+(650*650))); // Gives the reachable area for the ICDs
		double distance, distance_x, distance_y;


		vector<int>row;
		for (int i = 0; i < stocset.GetNumberOfICDs() ; i++)
		{
			for (int j = 0 ; j < stocset.GetNumberOfLDCs() ; j++)
			{ 
				distance_x = ldc_index[(j+1+stocset.GetNumberOfICDs())].GetCoordinateX()- icd_index[i+1].GetCoordinateX();
				distance_y = ldc_index[j+1+stocset.GetNumberOfICDs()].GetCoordinateY()- icd_index[i+1].GetCoordinateY();
				distance = sqrt((distance_x*distance_x) + (distance_y*distance_y));

				if ( distance < radius)
				{
				row.push_back(1);
				}
				else
				{
					row.push_back(0);
				}
			}
			arcs.allowed_arcs_icd.push_back(row);
			row.clear();
		}
		
		arcs.allowed_arcs_ldc.resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
		arcs.arc_cost_ldc.resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
		arcs.travel_time_ldc.resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
		
		for (int i = 0 ; i < stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs(); i++)
		{
			arcs.allowed_arcs_ldc[i].resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
			arcs.arc_cost_ldc[i].resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
			arcs.travel_time_ldc[i].resize(stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs());
			
			for (int j = 0 ; j < stocset.duplicated_LDCS_PODs + stocset.GetNumberOfLDCs(); j++)
			{
				arcs.allowed_arcs_ldc[i][j].resize(stocset.GetNumberOfVehicles());
				arcs.arc_cost_ldc[i][j].resize(stocset.GetNumberOfVehicles());
				arcs.travel_time_ldc[i][j].resize(stocset.GetNumberOfVehicles());
				
				for (int k = 0 ; k < stocset.GetNumberOfVehicles() ; k++)
				{
					arcs.allowed_arcs_ldc[i][j][k].assign(stocset.GetNumberOfScenarioTreeNodes(),0);
					arcs.arc_cost_ldc[i][j][k].assign(stocset.GetNumberOfScenarioTreeNodes(),0);
					arcs.travel_time_ldc[i][j][k].assign(stocset.GetNumberOfScenarioTreeNodes(),0);
				}
			}
		 }

		CreateStochArcs(stocset, icd_index, ldc_index, podder, earthquake, arcs);

///////Creating text files
		CreateStochFile(stocset, earthquake, icd_index, ldc_index, podder, arcs);
		
	
	return 0;

}
