
#ifndef STOCHASTIC_H
#define STOCHASTIC_H

#include <vector>

class Stochastic
{
private:
	int number_of_commodities;
	int number_of_utility_intervals;
	int number_of_vehicles;
	int number_of_ICDs;
	int number_of_LDCs;		//This set is not explicit formulated in math.model
	int number_of_PODs;		//This set is not explicit formulated in math.model
	int number_of_nodes;
	int vehicle_capacity_icd;
	int budget;
	int max_convoy_time;
	int number_of_scenario_tree_nodes;
	int number_of_stages;
	double number_of_possibilities_in_2; //This set is not explicit formulated in math.model
	double number_of_possibilities_in_3;
public:
	//variables
	std::vector<int>commodity_size;
	std::vector<int>commodity_cost;
	std::vector<int>vehicle_capacity;
	std::vector<int>vehicle_size; // is sat by the program
	
	double budget_penalty;
	int duplicated_LDCs;
	int duplicated_PODs;
	int duplicated_LDCS_PODs;
	int number_of_LDCs_PODs;

	//Set functions
	void SetNumberOfCommodities(int noc){number_of_commodities = noc;}
	void SetNumberOfUtilityIntervals(int noui){number_of_utility_intervals = noui;}
	void SetNumberOfVehicles(int nov){number_of_vehicles = nov;}
	void SetNumberOfICDs(int noi){number_of_ICDs = noi;}
	void SetNumberOfLDCs(int nol){number_of_LDCs = nol;}
	void SetNumberOfPODs(int nop){number_of_PODs = nop;}
	void SetNumberOfNodes();	
	void SetVehicleCapacityICD(int vci){vehicle_capacity_icd = vci;}
	void SetCommoditySize(int a);
	void SetCommodityCost(int cost);
	void SetBudget(int bud){budget = bud;}
	void SetConvoyTime(int time){max_convoy_time = time;}
	void SetVehicleCapacity(int vecap);
	void SetVehicleSize(int vsize);
	void SetNumberOfStages(){number_of_stages = 3;}
	void SetNumberOfScenarioTreeNodes();
	void SetNumberOfPoss2(double poss1){number_of_possibilities_in_2 = poss1;}
	void SetNumberOfPoss3(double
		poss2){number_of_possibilities_in_3 = poss2;}
	
	
	//Get functions
	int GetNumberOfCommodities(){return number_of_commodities;}
	int GetNumberOfUtilityIntervals(){return number_of_utility_intervals;}
	int GetNumberOfNodes(){return number_of_nodes;}
	int GetNumberOfVehicles(){return number_of_vehicles;}
	int GetNumberOfICDs(){return number_of_ICDs;}
	int GetNumberOfLDCs(){return number_of_LDCs;}
	int GetNumberOfPODs(){return number_of_PODs;}
	int GetVehicleCapacityICD(){return vehicle_capacity_icd;}
	int GetBudget(){return budget;}
	int GetMaxConvoyTime(){return max_convoy_time;}
	int GetNumberOfStages(){return number_of_stages;}
	int GetNumberOfScenarioTreeNodes(){return number_of_scenario_tree_nodes;}
	double GetNumberOfPossibilitiesIn2(){return number_of_possibilities_in_2;}
	double GetNumberOfPossibilitiesIn3(){return number_of_possibilities_in_3;}
};



class Arcs
{
public:
	std::vector<std::vector<int> >allowed_arcs_icd;
	std::vector<std::vector<std::vector<std::vector<int> > > >allowed_arcs_ldc;
	std::vector<std::vector<std::vector<std::vector<int> > > >arc_cost_ldc;
	std::vector<std::vector<std::vector<std::vector<int> > > >travel_time_ldc;

	std::vector<std::vector<std::vector<int> > >allowed_vehicle_types;
	std::vector<std::vector<int> > arc_quality;
	std::vector<std::vector<std::vector<int> > >arc_capacity;

	int arc_cost_icd[4][100];
	int travel_time_icd[4][100];
	
	std::vector<std::vector<int> >possible_arcs_ldc_pod1;

};




#endif